# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import logging

from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String, Multiple

logger = logging.getLogger("driverng.illumina")


class IlluminaResultBase(Splitted):

    SEPARATOR = "\t"


class IlluminaResult(object):
    FIELDS = [
        String("batch_name"),
        String("sample_barcode"),
        String("sample_type"),
        Multiple("results", (String("resu"),), separator="\t"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            if isinstance(field, Multiple):
                setattr(self, field.name, line[index:])
                break
            else:
                setattr(self, field.name, line[index])


class IlluminaResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: IlluminaResult}

    def __init__(self, separateur):
        self.separateur = separateur
        return super(IlluminaResultsMultiple, self).__init__()

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str("\t"))
        for line in reader:
            if not len(line):
                continue
            if len(line) >= 5:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "illumina", "label": "ILLUMINA CSV"}

    OPTIONS = [
        MessageV2.Option(
            "illumina_separateur", label="Séparateur de champs", default="\t"
        )
    ]

    def __init__(self, **kwargs):
        self.separateur = kwargs.pop("illumina_separateur", "\t")
        return super(Driver, self).__init__(**kwargs)

    def parse(self, data):
        obj = IlluminaResultsMultiple(self.separateur)
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        RESULTS_CODES = ()
        for m in message:
            if not m.sample_barcode:
                continue
            if m.batch_name == "batch_name" and m.sample_barcode == "sample_barcode":
                RESULTS_CODES = m.results
                continue
            if RESULTS_CODES:
                sample = worklist.get_or_create(
                    sample_id=m.sample_barcode, type="patient"
                )
                for index, result in enumerate(m.results):
                    if RESULTS_CODES[index] and result:
                        if RESULTS_CODES[index] == "ff" and result[-1] == "%":
                            resu = result[:-1]
                        else:
                            resu = result
                        sample.results.add(
                            code=RESULTS_CODES[index].strip(), value=resu
                        )
        return worklist
